#!/usr/bin/env python

#########################################################################
#                                                                       #
# Apathy - A telepathy based communications client for mobile platforms #
# Copyright (C) 2009 Alasdair MacLeod <alasdair.macleod@gmail.com>      #
# Copyright 2012 Kai Huuhko <kai@kaihuuhko.com>                         #
#                                                                       #
# This program is free software: you can redistribute it and/or modify  #
# it under the terms of the GNU General Public License as published by  #
# the Free Software Foundation, either version 3 of the License, or     #
# (at your option) any later version.                                   #
#                                                                       #
# This program is distributed in the hope that it will be useful,       #
# but WITHOUT ANY WARRANTY; without even the implied warranty of        #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
# GNU General Public License for more details.                          #
#                                                                       #
# You should have received a copy of the GNU General Public License     #
# along with this program.  If not, see <http://www.gnu.org/licenses/>. #
#                                                                       #
#########################################################################

import os.path
import logging
from getpass import getuser
import mimetypes
import tempfile

import elementary as elm

from ErrorDialog import ErrorDialog
from Globals import *

from gi.repository import GObject
from gi.repository import TelepathyGLib as Tp

class AccountEditor(object):
    def __init__(self, parent, pager, am, account=None):
        self.parentwin = parent
        self.pager = pager

        self.bt_protocol = None
        self.account = account
        self.am = am
        self.protos = {}
        self.cms = {}

        self.box = box = elm.Box(parent)

        self.protocol_list = protocol_list = elm.Hoversel(box)
        protocol_list.text= _("Select protocol").encode("utf-8")
        self.populate_protocol_list(protocol_list)
        box.pack_end(protocol_list)
        protocol_list.show()

        self.box1 = box1 = elm.Box(box)
        box1.size_hint_weight_set(1.0, 1.0)
        box1.size_hint_align_set(-1.0, -1.0)
        box1.show()

        scroller = elm.Scroller(box)
        scroller.size_hint_weight_set(1.0, 1.0)
        scroller.size_hint_align_set(-1.0, -1.0)
        scroller.content_set(box1)
        scroller.page_relative_set(0.0, 0.0)
        scroller.bounce_set(False, False)
        scroller.show()
        box.pack_end(scroller)

        self.box2 = box2 = elm.Box(box)
        box2.size_hint_weight_set(1.0, 0.0)
        box2.size_hint_align_set(-1.0, 0.0)
        box2.horizontal_set(True)
        box2.show()
        box.pack_end(box2)

        self.btn_save_settings = btn_save_settings = elm.Button(box)
        btn_save_settings.text_set(_("Save Settings").encode("utf-8"))
        btn_save_settings.size_hint_weight_set(1.0, 0.0)
        btn_save_settings.size_hint_align_set(-1.0, 0.0)
        btn_save_settings.callback_clicked_add(self.save)
        self.box2.pack_end(btn_save_settings)
        btn_save_settings.show()

        xbtn = elm.Button(box)
        xbtn.size_hint_weight_set(1.0, 0.0)
        xbtn.size_hint_align_set(-1.0, 0.0)
        xbtn.callback_clicked_add(self._close)
        xbtn.text_set(_("Close").encode("utf-8"))
        box2.pack_end(xbtn)
        xbtn.show()

    def populate_protocol_list(self, component):
        logging.debug("Populating protocol list")
        daemon = Tp.DBusDaemon.dup()
        Tp.list_connection_managers_async(daemon, self.got_cms, component)

    def got_cms(self, spam, result, component):
        cms = Tp.list_connection_managers_finish(result)

        for cm in cms:
            self.cms[cm.get_name()] = cm
            protos = cm.dup_protocols()
            for proto in protos:
                self.protos[proto.get_name()] = proto
                ic = elm.Icon(self.box)
                ic.standard_set("im-"+proto.get_name())
                component.item_add(str.capitalize(proto.get_name()+" ("+cm.get_name()+")"), "im-"+proto.get_name(), elm.ELM_ICON_STANDARD, self._protocol_selected, proto, cm)

        if self.account:
            self.populate_data(self.account)

    def populate_data(self, account):
        self.account = account
        cm = account.get_connection_manager()
        proto = account.get_protocol()
        self._protocol_selected(None, None, self.protos[proto], self.cms[cm])

        account.get_avatar_async(self.got_avatar, None)

        if proto == "jabber":
            self.jabber_service_selected(self.props[Tp.IFACE_ACCOUNT+".Service"], None, account.get_service())
        elif proto == "irc":
            self.irc_service_selected(self.props[Tp.IFACE_ACCOUNT+".Service"], None, account.get_service())

        self.dname.text_set(account.get_display_name())

        self.props[Tp.IFACE_ACCOUNT+".Icon"].text_set(account.get_icon_name())
        self.props[Tp.IFACE_ACCOUNT+".Nickname"].text_set(account.get_nickname())
        self.props[Tp.IFACE_ACCOUNT+".Enabled"].state_set(account.is_enabled())
        self.props[Tp.IFACE_ACCOUNT+".ConnectAutomatically"].state_set(account.get_connect_automatically())

        self.auto_presence_selected(None, None, account.get_automatic_presence())

        for param, v in self.params.iteritems():
            params = account.get_parameters()
            if param in params:
                if type(v).__name__ == "Check":
                    self.params[param].state_set(params[param])
                elif type(v).__name__ == "Entry":
                    self.params[param].text_set(params[param])
                else:
                    logging.error("Unknown field %s" % (v))

    def got_avatar(self, account, result, user_data):
        data = account.get_avatar_finish(result)
        if data:
            avatar = tempfile.NamedTemporaryFile(delete=False)
            avatar.write(data)
            avatar.close()
            self.avatar.file_set(avatar.name)

    def _protocol_selected(self, obj, data, proto, cm):
        self.proto = proto
        self.cm = cm
        self.protocol_list.text_set(str.capitalize(proto.get_name()+" ("+cm.get_name()+")"))
        self.box1.clear()

        self.props = {}
        self.params = {}
        self.changed = {}

        params = proto.dup_params()

        # Avatar
        self.avatar = avatar = elm.Photo(self.box)
        avatar.size_set(100)
        avatar.callback_clicked_add(self.avatar_clicked)

        avatarframe = elm.Frame(self.box)
        avatarframe.text_set(_("Avatar").encode("utf-8"))
        avatarframe.size_hint_align_set(0.5, -1.0)
        avatarframe.content_set(avatar)
        self.box1.pack_end(avatarframe)
        avatar.show()
        avatarframe.show()

        # Service
        proto_name = proto.get_name()
        if proto_name == "jabber" or proto_name == "irc":
            self.props[Tp.IFACE_ACCOUNT+".Service"] = service = elm.Hoversel(self.box)
            service.text_set(_("Select").encode("utf-8"))
            servframe = elm.Frame(self.box)
            servframe.size_hint_align_set(-1.0, -1.0)
            servframe.text_set(_("Service").encode("utf-8"))
            servframe.content_set(service)
            self.box1.pack_end(servframe)
            servframe.show()
            service.show()

            proto_name = proto.get_name()
            if proto_name == "jabber":
                for s in ["jabber", "google-talk", "ovi-chat", "facebook", "lj-talk", "windows-live"]:
                    service.item_add(str.capitalize(s), "im-"+s, elm.ELM_ICON_STANDARD, self.jabber_service_selected, s)
            elif proto_name == "irc":
                for s in ["Freenode", "IRCNet", "GimpNet"]:
                    service.item_add(s, None, elm.ELM_ICON_NONE, self.irc_service_selected, s)

        # Display Name
        dnameframe = elm.Frame(self.box)
        dnameframe.text_set(_("Display Name").encode("utf-8"))
        dnameframe.size_hint_align_set(-1.0, -1.0)
        self.dname = dname = elm.Entry(self.box)
        dname.single_line_set(True)
        dnameframe.content_set(dname)
        dname.show()
        self.box1.pack_end(dnameframe)
        dnameframe.show()
        dname.text_set(getuser()+" ("+proto.get_name()+")")

        # Entries: Nickname, Icon
        for name, title in {Tp.IFACE_ACCOUNT+".Nickname": _("Nickname").encode("utf-8"), Tp.IFACE_ACCOUNT+".Icon": _("Icon").encode("utf-8")}.iteritems():
            frame = elm.Frame(self.box)
            frame.text_set(title)
            frame.size_hint_align_set(-1.0, -1.0)
            self.props[name] = entry = elm.Entry(self.box)
            entry.single_line_set(True)
            frame.content_set(entry)
            entry.show()
            self.box1.pack_end(frame)
            frame.show()

        self.props[Tp.IFACE_ACCOUNT+".Icon"].text_set("im-"+proto.get_name())

        # Checks: Enabled, ConnectAutomatically
        for name, title in {Tp.IFACE_ACCOUNT+".Enabled": _("Enabled").encode("utf-8"), Tp.IFACE_ACCOUNT+".ConnectAutomatically": _("Connect Automatically").encode("utf-8")}.iteritems():
            frame = elm.Frame(self.box)
            frame.text_set(title)
            frame.size_hint_align_set(-1.0, -1.0)
            self.props[name] = check = elm.Check(self.box)
            frame.content_set(check)
            check.show()
            self.box1.pack_end(frame)
            frame.show()

        self.props[Tp.IFACE_ACCOUNT+".Enabled"].state_set(True)

        # TODO: Automatic Presence
        apres = elm.Hoversel(self.box)
        apres.text_set(_("Select"))
        apresframe = elm.Frame(self.box)
        apresframe.size_hint_align_set(-1.0, -1.0)
        apresframe.text_set(_("Automatic Presence"))
        apresframe.content_set(apres)
        #self.box1.pack_end(apresframe)
        #apresframe.show()
        #apres.show()

        paramhead = elm.Label(self.box)
        paramhead.text_set("<br/>"+_("Account parameters:").encode("utf-8")+"<br/>")
        paramhead.show()
        self.box1.pack_end(paramhead)

        # Parameters
        for param in params:
            if param.is_dbus_property():
                continue

            pname = param.get_name()
            defaultvalue = GObject.Value()

            frame = elm.Frame(self.box)
            frame.size_hint_align_set(-1.0, -1.0)

            frametext = "%s" % (pname.rsplit(".", 1)[-1])
            if param.is_required():
                frametext = frametext+" "+_("REQUIRED")

            ptype = param.get_dbus_signature()

            if ptype == "b":
                entry = self.params[pname] = elm.Check(self.box)
                entry.callback_changed_add(lambda x, pname=pname: self._changed(x, pname))
                if param.get_default(defaultvalue):
                    pdefault = defaultvalue.get_boolean()
                    entry.state_set(pdefault)
                    frametext = frametext+" - "+_("Default: %s") % (str(pdefault))

            elif ptype == "s":
                entry = self.params[pname] = elm.Entry(self.box)
                entry.callback_changed_user_add(lambda x, pname=pname: self._changed(x, pname))
                entry.single_line_set(True)
                if param.is_secret():
                    entry.password_set(True)
                if param.get_default(defaultvalue):
                    pdefault = defaultvalue.get_string()
                    entry.entry_set(pdefault)
                    frametext = frametext+" - "+_("Default: %s") % (pdefault)

            elif ptype in ["n", "q", "i", "u", "x", "t"]:
                entry = self.params[pname] = elm.Spinner(self.box)
                entry.callback_changed_add(lambda x, pname=pname: self._changed(x, pname))
                if "port" in pname:
                    entry.min_max_set(0, 65535)
                else:
                    entry.min_max_set(0, 100000)
                if param.get_default(defaultvalue):
                    if ptype in ["n", "i", "x"]:
                        pdefault = defaultvalue.get_int()
                    else:
                        pdefault = defaultvalue.get_uint()
                    entry.value_set(pdefault)
                    frametext = frametext+" - "+_("Default: %s") % (str(pdefault))

            #elif ptype == "as":
                #Multiline entry? Comma-delimited? List with buttons?
                #entry = self.params[pname] = List(self.box)

            else:
                logging.debug("Unknown parameter type %s in %s" % (ptype, pname))
                entry = self.params[pname] = elm.Entry(self.box)
                entry.callback_changed_user_add(lambda x, pname=pname: self._changed(x, pname))
                entry.single_line_set(True)

            frame.text_set(frametext)

            entry.show()
            frame.content_set(entry)
            self.box1.pack_end(frame)
            frame.show()

    def jabber_service_selected(self, hoversel, item, service):
        hoversel.text_set(service.capitalize())
        if service:
            self.props[Tp.IFACE_ACCOUNT+".Icon"].text_set("im-"+service)
        else:
            self.props[Tp.IFACE_ACCOUNT+".Icon"].text_set("im-jabber")
        self.props[Tp.IFACE_ACCOUNT+".Service"] = service

    def irc_service_selected(self, hoversel, item, service):
        hoversel.text_set(service.capitalize())
        self.props[Tp.IFACE_ACCOUNT+".Service"] = service

    def auto_presence_selected(self, hoversel, item, presence):
        #TODO: Automatic Presence
        pass

    def avatar_clicked(self, obj):
        avatarwin = elm.InnerWindow(self.parentwin)

        avatarsel = elm.Fileselector(avatarwin)
        avatarsel.is_save_set(False)
        avatarsel.expandable_set(True)
        avatarsel.callback_done_add(self.avatar_selected, avatarwin)

        avatarwin.content_set(avatarsel)
        avatarsel.show()
        avatarwin.activate()

    def avatar_selected(self, obj, selected, avatarwin):
        if selected:
            with open(selected, "r") as f:
                avatar = bytearray(f.read())
            mimetype = mimetypes.guess_type(selected)[0]

            if self.check_avatar(mimetype, avatar):
                self.avatar.file_set(selected)
                self.avatardata = [bytearray(avatar), mimetype]
        obj.delete()
        avatarwin.delete()

    def check_avatar(self, mimetype, avatar):
        req = self.proto.get_avatar_requirements()

        if not req:
            req = Tp.AvatarRequirements.new(
                ("image/png", "image/jpeg", "image/gif"),
                32, #min
                32,
                64, #recommended
                64,
                96, #max
                96,
                32768 #max file
            )

        warn_file_size = 8192
        valid = True

        # Mime type checks
        if not mimetype in req.supported_mime_types:
            dialog = ErrorDialog(self.parentwin, _("Error").encode("utf-8"), _("Your avatar file is of wrong type, %s, the allowed types are %s") % (mimetype, str(req.supported_mime_types)))
            valid = False

        # File Size checks
        if len(avatar)>req.maximum_bytes:
            dialog = ErrorDialog(self.parentwin, _("Error").encode("utf-8"), _("Your avatar file is too large, %d bytes, the limit is %d bytes") % (len(avatar), req.maximum_bytes))
            valid = False
        elif len(avatar)>warn_file_size:
            dialog = ErrorDialog(self.parentwin, _("Warning").encode("utf-8"), _("Your avatar file is of size %d bytes, which is larger than the recommended size %d bytes. Be aware that this account may not be able to set its presence correctly.") % (len(avatar), warn_file_size))

        return valid

    def _changed(self, x, key):
        logging.debug("Entry %s changed" % (key.rsplit(".", 1)[-1]))
        self.changed[key] = x

    def save(self, data):
        try: self.proto
        except:
            logging.info("No Protocol Selected")
            errdlg = ErrorDialog(self.parentwin, _("Error").encode("utf-8"), _("No Protocol Selected").encode("utf-8"))
            return

        account = {}
        account["displayname"] = self.dname.entry_get()
        account["props"] = {}
        account["params"] = {}

        for k, v in self.props.iteritems():
            if type(v).__name__ == "Check":
                logging.debug("prop check %s" % (k))
                account["props"][k] = v.state_get()
            elif type(v).__name__ == "Entry":
                prop = v.entry_get().replace("<br>", "")
                logging.debug("prop entry %s" % (k))
                account["props"][k] = str(prop)
            elif type(v).__name__ == "str":
                logging.debug("prop string %s" % (k))
                account["props"][k] = v
            elif type(v).__name__ == "tuple":
                logging.debug("prop list %s" % (k))
                account["props"][k] = v
            else:
                logging.debug("Unknown prop type %s in %s" % (type(v).__name__, k))

        for k, v in self.changed.iteritems():
            k = k.rsplit(".", 1)[-1]
            if type(v).__name__ == "Check":
                logging.debug("param check %s" % (k))
                account["params"][k] = v.state_get()
            elif type(v).__name__ == "Entry":
                param = v.entry_get().replace("<br>", "")
                logging.debug("param entry %s" % (k))
                account["params"][k] = str(param)
            elif type(v).__name__ == "Spinner":
                param = v.value_get()
                logging.debug("param spinner %s" % (k))
                account["params"][k] = int(param)
            #elif type(v).__name__ == "":
            else:
                logging.debug("Unknown param type %s in %s" % (type(v).__name__, k))

        logging.info("Saving account for Display Name %s" % (account["displayname"]))

        if self.account:
            #We are editing an account
            apaccount = self.account
            apaccount.set_display_name_async(account["displayname"], self._display_name_set, None)
            apaccount.set_enabled_async(account["props"][Tp.IFACE_ACCOUNT+".Enabled"], self._prop_set, None)
            apaccount.set_icon_name_async(account["props"][Tp.IFACE_ACCOUNT+".Icon"], self._prop_set, None)
            try: account["props"][Tp.IFACE_ACCOUNT+".Service"]
            except KeyError:
                pass
            else:
                apaccount.set_service_async(account["props"][Tp.IFACE_ACCOUNT+".Service"], self._prop_set, None)
            #TODO: apaccount.set_automatic_presence_async(account["props"]["autopres"], self._prop_set, None)
            apaccount.set_connect_automatically_async(account["props"][Tp.IFACE_ACCOUNT+".ConnectAutomatically"], self._prop_set, None)
            apaccount.set_nickname_async(account["props"][Tp.IFACE_ACCOUNT+".Nickname"], self._prop_set, None)

            try: self.avatardata
            except AttributeError:
                pass
            else:
                avatar, mimetype = self.avatardata
                apaccount.set_avatar_async(avatar, mimetype, self._avatar_set, None)

            if account["params"]:
                # XXX: update_parameters(_vardict)_async is broken by https://bugs.freedesktop.org/show_bug.cgi?id=59757
                pu = elm.Popup(self.box)
                pu.part_text_set("title,text", "Account parameter updates disabled")
                pu.text = "Updating the account parameters has been temporally disabled pending a fix to the underlaying library. As a workaround delete and recreate your account."
                pub = elm.Button(self.box)
                pub.text = "OK"
                pub.callback_clicked_add(self._close)
                pu.part_content_set("button1", pub)
                pu.show()
                #apaccount.update_parameters_async(account["params"], "", self._prop_set, None)
            logging.info("Account %s updated" % (account["displayname"]))
            #self._close()
        else:
            self.am.create_account_async(self.cm.get_name(), self.proto.get_name(), self.dname.entry_get(), account["params"], account["props"], self.account_created, None)

    def account_created(self, am, result, user_data):
        try:
            account = am.create_account_finish(result)
        except GObject.GError as gerror:
            logging.exception("Error while creating account")
            ErrorDialog(self.parentwin, _("Error").encode("utf-8"), gerror.message)
        else:
            logging.info("Account %s created" % (account.get_display_name()))
            try: self.avatardata
            except: pass
            else:
                avatar, mimetype = self.avatardata
                account.set_avatar_async(avatar, mimetype, self._avatar_set, None)
            self._close()

    def _display_name_set(self, account, result, user_data):
        if not account.set_display_name_finish(result):
            errordlg = ErrorDialog(self.parentwin, _("Error").encode("utf-8"), "Could not set Display Name")

    def _avatar_set(self, account, result, user_data):
        if not account.set_avatar_finish(result):
            errordlg = ErrorDialog(self.parentwin, _("Error").encode("utf-8"), "Could not set Avatar")

    def _prop_set(self, account, result, user_data):
        pass

    def _close(self, *args):
        self.pager.item_pop()
