#!/usr/bin/env python

#########################################################################
#                                                                       #
# Apathy - A telepathy based communications client for mobile platforms #
# Copyright (C) 2009  Michael Sheldon <mike@mikeasoft.com>              #
# Copyright 2012 Kai Huuhko <kai@kaihuuhko.com>                         #
#                                                                       #
# This program is free software: you can redistribute it and/or modify  #
# it under the terms of the GNU General Public License as published by  #
# the Free Software Foundation, either version 3 of the License, or     #
# (at your option) any later version.                                   #
#                                                                       #
# This program is distributed in the hope that it will be useful,       #
# but WITHOUT ANY WARRANTY; without even the implied warranty of        #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
# GNU General Public License for more details.                          #
#                                                                       #
# You should have received a copy of the GNU General Public License     #
# along with this program.  If not, see <http://www.gnu.org/licenses/>. #
#                                                                       #
#########################################################################

import os.path, gettext

from Accounts import Accounts
from Contacts import Contacts
from Globals import *

import ecore, dbus, e_dbus
import elementary as e

from gi.repository import GObject
GObject.threads_init()
from gi.repository import TelepathyGLib as Tp
#Tp.debug_set_flags("all")

class Apathy(e.Window):
    def __init__(self, log):
        e.Window.__init__(self, "apathy", e.ELM_WIN_BASIC)
        
        self.title_set("Apathy")
        self.callback_delete_request_add(self.quit)

        bg = e.Background(self)
        bg.size_hint_weight_set(1.0, 1.0)
        self.resize_object_add(bg)
        bg.show()
                
        self.flip = flip = e.Flip(self)
        self.contacts = contacts = Contacts(self)
        self.accounts = accounts = Accounts(self)
        flip.content_front_set(contacts)
        flip.content_back_set(accounts)
        flip.size_hint_weight_set(1.0, 1.0)
        flip.size_hint_align_set(-1.0, -1.0)
        self.resize_object_add(flip)
        flip.show()
        
        self.resize(480, 640)
        self.show()
        
    def quit(self, *args):
        log.info("Apathy is exiting")
        e.exit()

if __name__ == "__main__":
    if not os.path.exists(USER_PATH):
        os.mkdir(USER_PATH, 0700)
    if not os.path.exists(IMAGE_PATH):
        os.mkdir(IMAGE_PATH, 0777)

    gettext.install("apathy", "/usr/share/locale", unicode=1)

    e.init()
    dbus_mainloop = e_dbus.DBusEcoreMainLoop()
    dbus.set_default_main_loop(dbus_mainloop)

    apathy = Apathy(log)
    
    ecore.main_loop_glib_integrate()

    e.run()
    e.shutdown()
