#!/usr/bin/env python

#########################################################################
#                                                                       #
# Apathy - A telepathy based communications client for mobile platforms #
# Copyright (C) 2009  Michael Sheldon <mike@mikeasoft.com>              #
# Copyright 2012 Kai Huuhko <kai@kaihuuhko.com>                         #
#                                                                       #
# This program is free software: you can redistribute it and/or modify  #
# it under the terms of the GNU General Public License as published by  #
# the Free Software Foundation, either version 3 of the License, or     #
# (at your option) any later version.                                   #
#                                                                       #
# This program is distributed in the hope that it will be useful,       #
# but WITHOUT ANY WARRANTY; without even the implied warranty of        #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
# GNU General Public License for more details.                          #
#                                                                       #
# You should have received a copy of the GNU General Public License     #
# along with this program.  If not, see <http://www.gnu.org/licenses/>. #
#                                                                       #
#########################################################################

from elementary import *
from evas import EVAS_ASPECT_CONTROL_VERTICAL

from gi.repository import TelepathyGLib as Tp

class ContactInfo(Box):
    def __init__(self, parent, contact):
        self.parentwin = parent
        Box.__init__(self, parent.window)

        self.size_hint_align_set(-1.0, -1.0)
        self.size_hint_weight_set(1.0, 1.0)

        ident = Label(self)
        ident.text_set("<br><b>%s</b><br>" % contact.get_identifier())
        self.pack_end(ident)
        ident.show()
        
        avatar = Photo(self)
        avatar.size_set(200)
        avatar_file = contact.get_avatar_file()
        if avatar_file:
            path = avatar_file.get_path()
            avatar.file_set(path)
        self.pack_end(avatar)
        avatar.show()
        
        textbox = Box(self)
        textcap = ""
        audiocap = ""
        audiovideocap = ""
        caps = contact.get_capabilities()
        if caps:
            if caps.supports_text_chats():
                textcap = _("Yes")
            if caps.supports_audio_call(Tp.HandleType.CONTACT):
                audiocap = _("Yes")
            if caps.supports_audio_video_call(Tp.HandleType.CONTACT):
                audiovideocap = _("Yes")
        states = {
            Tp.SubscriptionState.UNKNOWN: _("Unknown"),
            Tp.SubscriptionState.NO: _("No"),
            Tp.SubscriptionState.REMOVED_REMOTELY: _("Denied"),
            Tp.SubscriptionState.ASK: _("Waiting confirmation"),
            Tp.SubscriptionState.YES: _("Yes")
        }
        
        location = ""
        if contact.get_location():
            location = contact.get_location()
            
        blocked = _("No")
        if contact.is_blocked():
            blocked = _("Yes")
        
        texts = {
            _("Alias"): contact.get_alias().decode("utf-8"),
            _("Status"): contact.get_presence_status().capitalize(),
            _("Message"): contact.get_presence_message().decode("utf-8"),
            _("Location"): location,
            _("Account"): contact.get_connection().get_account().get_display_name(),
            _("Subscribed to"): states[contact.get_subscribe_state()],
            _("Published to"): states[contact.get_publish_state()],
            _("Blocked"): blocked,
            _("Can send text"): textcap,
            _("Can call"): audiocap,
            _("Can call with video"): audiovideocap,
        }
        for k, v in texts.iteritems():
            k = k.encode("ascii", "xmlcharrefreplace")
            v = v.encode("ascii", "xmlcharrefreplace")
            label = Label(self)
            label.text_set("<b>%s:</b> %s" % (k, v))
            label.size_hint_align_set(0.0, 0.0)
            textbox.pack_end(label)
            label.show()
        
        self.pack_end(textbox)
        textbox.show()
        
        fill = Box(self)
        fill.size_hint_weight_set(1.0, 1.0)
        self.pack_end(fill)

        xbtn = Button(self)
        xbtn.text_set(_("Close"))
        xbtn.size_hint_weight_set(1.0, 0.0)
        xbtn.size_hint_align_set(-1.0, 1.0)
        xbtn.callback_clicked_add(self.close)
        self.pack_end(xbtn)
        xbtn.show()

    def close(self, button):
        self.parentwin.pager.item_pop()
