#!/usr/bin/env python

#########################################################################
#                                                                       #
# Apathy - A telepathy based communications client for mobile platforms #
# Copyright (C) 2009  Michael Sheldon <mike@mikeasoft.com>              #
# Copyright 2012 Kai Huuhko <kai@kaihuuhko.com>                         #
#                                                                       #
# This program is free software: you can redistribute it and/or modify  #
# it under the terms of the GNU General Public License as published by  #
# the Free Software Foundation, either version 3 of the License, or     #
# (at your option) any later version.                                   #
#                                                                       #
# This program is distributed in the hope that it will be useful,       #
# but WITHOUT ANY WARRANTY; without even the implied warranty of        #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
# GNU General Public License for more details.                          #
#                                                                       #
# You should have received a copy of the GNU General Public License     #
# along with this program.  If not, see <http://www.gnu.org/licenses/>. #
#                                                                       #
#########################################################################

import logging
from datetime import datetime
from time import time
from threading import Timer
import webbrowser

from evas import EVAS_ASPECT_CONTROL_VERTICAL
import elementary as elm

from Globals import *
from Parser import parse

from gi.repository import TelepathyGLib as Tp

class Conversation(object):
    def __init__(self, parent, channel):
        self.parentwin = parent.window
        self.contact = channel.get_target_contact()
        self.channel = channel

        global log
        log = logging.getLogger("apathy.handler")

        self.chat_state = "inactive"

        box = self.box = elm.Box(self.parentwin)
        box.size_hint_weight_set(1.0, 1.0)
        box.size_hint_align_set(-1.0, -1.0)

        self.scroll = elm.Scroller(self.box)
        self.scroll.size_hint_weight_set(1.0, 1.0)
        self.scroll.size_hint_align_set(-1.0, -1.0)
        self.scroll.bounce_set(False,True)
        box.pack_end(self.scroll)
        self.scroll.show()

        self.message_box = elm.Box(self.box)
        self.message_box.size_hint_weight_set(1.0, 1.0)
        self.message_box.size_hint_align_set(-1.0, -1.0)
        self.scroll.content_set(self.message_box)
        self.message_box.show()

        fr_entry = elm.Frame(self.box)
        fr_entry.text_set(_("Input:").encode("utf-8"))
        fr_entry.size_hint_align_set(0.0, 0.0)
        fr_entry.size_hint_align_set(-1.0, 0.0)

        self.text_entry = elm.Entry(self.box)
        self.text_entry.elm_event_callback_add(self.entry_event)
        self.text_entry.focus_set(True)

        fr_entry.content_set(self.text_entry)
        box.pack_end(fr_entry)
        self.text_entry.show()
        fr_entry.show()

        box1 = elm.Box(self.box)
        box1.horizontal_set(True)
        box1.size_hint_weight_set(1.0, 0.0)
        box1.size_hint_align_set(-1.0, 0.0)
        box.pack_end(box1)

        self.entry_mode = entry_mode = elm.Check(self.box)
        entry_mode.text_set(_("Quick entry mode").encode("utf-8"))
        box1.pack_end(entry_mode)
        entry_mode.show()

        self.emotes = emotes = elm.Check(self.box)
        emotes.text_set(_("Emoticons").encode("utf-8"))
        box1.pack_end(emotes)
        emotes.show()

        send_button = elm.Button(self.box)
        send_button.text_set(_("Send").encode("utf-8"))
        send_button.size_hint_weight_set(1.0, 0.0)
        send_button.size_hint_align_set(-1.0, 0.0)
        send_button.callback_clicked_add(self.send_clicked)
        box1.pack_end(send_button)
        send_button.show()

        box1.show()
        box.show()

        channel.connect('message-received', self.message_received)
        channel.connect('message-sent', self.message_sent)
        #channel.connect('send-error', self.send_error_cb)
        channel.connect("chat-state-changed", self.chat_state_changed)

        messages = channel.get_pending_messages()
        for message in messages:
            self.message_received(channel, message)

    def request_channel_error_cb(self, exception):
        log.error("Error" % (str(exception)))

    def add_bubble(self, text, timestamp, sender, corner, avatar = None):
        timestr = datetime.fromtimestamp(timestamp).strftime("%c")
        bb = elm.Bubble(self.box)
        bb.pos_set(corner)
        bb.text_set(sender)
        bb.part_text_set("info", timestr)
        bb.size_hint_weight_set(0.0, 0.0)
        bb.size_hint_align_set(-1.0, 0.0)
        ab = elm.Entry(self.box)
        ab.callback_anchor_clicked_add(self.anchor_clicked)
        ab.callback_anchor_hover_opened_add(self.anchor_hover_opened)
        ab.editable_set(False)
        ab.text_set(text)
        ab.anchor_hover_style_set("popup")
        ab.anchor_hover_parent_set(self.parentwin)
        bb.part_content_set("default", ab)
        if avatar:
            bb.part_content_set("icon", avatar)

        self.message_box.pack_end(bb)
        bb.show()
        self.scroll.region_show(0, self.scroll.child_size_get()[1] - 0, 100, 100)

    def anchor_clicked(self, obj, event_info):
        log.debug("Clicked on anchor %s" % (event_info.name))
        self.anchor_name = event_info.name

    def anchor_hover_opened(self, obj, event_info):
        log.debug(event_info)
        btn = elm.Button(obj)
        parts = self.anchor_name.partition(":")
        if parts[0] == "url":
            btn.text_set("Open in browser")
            btn.callback_clicked_add(lambda x: webbrowser.open(parts[2]))
            btn2 = elm.Button(obj)
            btn2.text_set("Open internally")
            btn2.callback_clicked_add(self.open_web, parts[2])
            btn2.callback_clicked_add(lambda x: obj.anchor_hover_end())
            event_info.hover.part_content_set("top", btn2)
            btn2.show()
        elif parts[0] == "mailto":
            btn.text_set("Send mail")
            btn.callback_clicked_add(lambda x: webbrowser.open(self.anchor_name))
        btn.callback_clicked_add(lambda x: obj.anchor_hover_end())
        btn.show()
        event_info.hover.part_content_set("middle", btn)

    def open_web(self, obj, url):
        box = elm.Box(self.parentwin)
        web = elm.Web(self.parentwin)
        inwin = elm.InnerWindow(self.parentwin)
        xbtn = elm.Button(self.parentwin)

        web.size_hint_weight_set(1.0, 1.0)
        web.size_hint_align_set(-1.0, -1.0)

        web.uri_set(url)
        xbtn.text_set(_("Close").encode("utf-8"))
        xbtn.callback_clicked_add(lambda x: inwin.delete())

        box.pack_end(web)
        box.pack_end(xbtn)
        inwin.content_set(box)

        web.show()
        xbtn.show()
        box.show()
        inwin.show()

    def chat_state_changed(self, channel, contact, state, *args):
        ic = elm.Icon(self.parentwin)
        ic.file_set(os.path.join(IMAGE_PATH, "apathy.png"))
        ic.size_hint_aspect_set(EVAS_ASPECT_CONTROL_VERTICAL, 1, 1)

        states = {
            Tp.ChannelChatState.GONE: "left the chat",
            Tp.ChannelChatState.INACTIVE: "has not been active",
            Tp.ChannelChatState.ACTIVE: "is active",
            Tp.ChannelChatState.PAUSED: "paused writing message",
            Tp.ChannelChatState.COMPOSING: "is writing message",
        }
        log.debug("%s, %s, %s" % (channel, contact, states[state]))
        if state == Tp.ChannelChatState.GONE:
            self.add_bubble("<em>%s %s</em>" % (self.contact.get_alias(), states[state]), time(), self.contact.get_alias(), elm.ELM_BUBBLE_POS_TOP_LEFT, ic)
        #TODO: chat states

    def entry_event(self, obj, entry, something, event, *args):
        if event.keyname == "Return" and self.entry_mode.state_get():
            self.send(entry.entry_get().replace("<br/>", ""))
        else:
            if self.chat_state == "composing":
                self.compose_timer.cancel()
            elif self.chat_state == "paused":
                self.pause_timer.cancel()
            self.compose_timer = Timer(2.0, self.timer_stopped)
            self.compose_timer.start()
            if not self.chat_state == "composing":
                self.channel.set_chat_state_async(Tp.ChannelChatState.COMPOSING, None, None)
                self.chat_state = "composing"
        return True

    def paused(self, channel, result, *args):
        self.pause_timer = Timer(2.0, self.timer_stopped)
        self.pause_timer.start()

    def timer_stopped(self, *args):
        if self.chat_state == "composing":
            self.channel.set_chat_state_async(Tp.ChannelChatState.PAUSED, self.paused, None)
            self.chat_state = "paused"
        elif self.chat_state == "paused":
            self.channel.set_chat_state_async(Tp.ChannelChatState.ACTIVE, None, None)
            self.chat_state = "active"

    def send_clicked(self, obj, *args):
        message = self.text_entry.entry_get()
        self.send(message)

    def send(self, message):
        if not len(message.replace("<br/>", "\n").strip()) == 0:
            text = self.text_entry.markup_to_utf8(message)
            message = Tp.ClientMessage.new_text(Tp.ChannelTextMessageType.NORMAL, text)
            self.channel.send_message_async(message, 0, lambda a, b, c: 0, None)
        self.text_entry.entry_set("")
        self.text_entry.focus_set(True)

        self.compose_timer.cancel()
        try:
            self.pause_timer.cancel()
        except:
            pass
        self.channel.set_chat_state_async(Tp.ChannelChatState.ACTIVE, None, None)
        self.chat_state = "active"

    def message_received(self, channel, message):
        text, flags = message.to_text()
        timestamp = message.get_received_timestamp()

        log.debug(text)
        log.debug(type(text))
        types = {
            Tp.ChannelTextMessageType.NORMAL: "Normal message",
            Tp.ChannelTextMessageType.ACTION: "User action",
            Tp.ChannelTextMessageType.NOTICE: "Notice",
            Tp.ChannelTextMessageType.AUTO_REPLY: "Automatically generated reply",
            Tp.ChannelTextMessageType.DELIVERY_REPORT: "Delivery report"
        }
        log.debug(types[message.get_message_type()])

        if message.get_message_type() == Tp.ChannelTextMessageType.DELIVERY_REPORT:
            channel.ack_message_async(message, lambda a, b, c: 0, None)
            return

        try:
            avatar_path = self.contact.get_avatar_file().get_path()
        except:
            avatar = None
        else:
            avatar = elm.Icon(self.parentwin)
            avatar.file_set(avatar_path)
            avatar.size_hint_aspect_set(EVAS_ASPECT_CONTROL_VERTICAL, 1, 1)

        text = parse(self.text_entry.utf8_to_markup(text), urls=False, emotes=self.emotes.state_get())
        log.debug(text)
        log.debug(type(text))
        self.add_bubble(text, timestamp, self.contact.get_alias(), elm.ELM_BUBBLE_POS_TOP_LEFT, avatar)

        if not self.parentwin.focus_get():
            notify(_("Message from %s").encode("utf-8") % (self.contact.get_alias()), text, "notification-information")
            set_urgent(self.parentwin, True)

        channel.ack_message_async(message, lambda a, b, c: 0, None)

    def message_sent(self, channel, message, flags, token, *args):
        text, flags = message.to_text()
        log.debug(text)
        timestamp = message.get_sent_timestamp()
        text = text.replace("\n", "<br/>")
        text = parse(text, urls=False, emotes=self.emotes.state_get())
        self.add_bubble(text, timestamp, "You", elm.ELM_BUBBLE_POS_TOP_RIGHT)

    def send_error_cb(self, error, timestamp, type, text):
        log.debug('Error %d sending message: %s' % (error, text))
