
#########################################################################
#                                                                       #
# Apathy - A telepathy based communications client for mobile platforms #
# Copyright (C) 2009  Michael Sheldon <mike@mikeasoft.com>              #
# Copyright 2012 Kai Huuhko <kai@kaihuuhko.com>                         #
#                                                                       #
# This program is free software: you can redistribute it and/or modify  #
# it under the terms of the GNU General Public License as published by  #
# the Free Software Foundation, either version 3 of the License, or     #
# (at your option) any later version.                                   #
#                                                                       #
# This program is distributed in the hope that it will be useful,       #
# but WITHOUT ANY WARRANTY; without even the implied warranty of        #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
# GNU General Public License for more details.                          #
#                                                                       #
# You should have received a copy of the GNU General Public License     #
# along with this program.  If not, see <http://www.gnu.org/licenses/>. #
#                                                                       #
#########################################################################

from os import getenv
import os.path
import ConfigParser

try:
    from gi.repository import Notify as LibNotify #clash with elm Notify
except ImportError:
    def notify(*args):
        pass
else:
    LibNotify.init("Apathy")
    def notify(title, msg, importance):
        n = LibNotify.Notification.new(title, msg, importance)
        n.set_timeout(3000)
        n.show()
        
try:
    import ecore.x
except ImportError:
    def set_urgent(*args):
        pass
else:
    from datetime import datetime
    def set_urgent(window, urgent):
        xid = window.xwindow_xid_get()
        xwin = ecore.x.Window_from_xid(xid)
        hints = list(xwin.icccm_hints_get())
        hints[6] = int(urgent) #Urgency hint
        xwin.icccm_hints_set(*hints)

USER_PATH = getenv("APATHY_USER_PATH")
IMAGE_PATH = getenv("APATHY_IMAGE_PATH")

if not USER_PATH:
    USER_PATH = os.path.expanduser(os.path.join("~", ".apathy"))
if not IMAGE_PATH:
    IMAGE_PATH = os.path.join(USER_PATH, "images")
    
def get_conf():
    conf = ConfigParser.SafeConfigParser()
    conf_path = os.path.join(USER_PATH, "apathy.conf")
    if not os.path.exists(USER_PATH):
        os.mkdir(USER_PATH, 0700)
    if not os.path.exists(os.path.join(conf_path)):
        conf.add_section("StatusMessages")
        conf.set("StatusMessages", "available", "http://launchpad.net/apathy")
        conf.set("StatusMessages", "away", "http://launchpad.net/apathy")
        conf.set("StatusMessages", "xa", "http://launchpad.net/apathy")
        conf.set("StatusMessages", "dnd", "http://launchpad.net/apathy")
        with open(conf_path, 'wb') as configfile:
            conf.write(configfile)
    conf.read(conf_path)
    return conf
