#!/usr/bin/env python

#########################################################################
#                                                                       #
# Apathy - A telepathy based communications client for mobile platforms #
# Copyright (C) 2009  Michael Sheldon <mike@mikeasoft.com>              #
# Copyright 2012 Kai Huuhko <kai@kaihuuhko.com>                         #
#                                                                       #
# This program is free software: you can redistribute it and/or modify  #
# it under the terms of the GNU General Public License as published by  #
# the Free Software Foundation, either version 3 of the License, or     #
# (at your option) any later version.                                   #
#                                                                       #
# This program is distributed in the hope that it will be useful,       #
# but WITHOUT ANY WARRANTY; without even the implied warranty of        #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
# GNU General Public License for more details.                          #
#                                                                       #
# You should have received a copy of the GNU General Public License     #
# along with this program.  If not, see <http://www.gnu.org/licenses/>. #
#                                                                       #
#########################################################################

from elementary import *
from evas import EVAS_ASPECT_CONTROL_VERTICAL

from Globals import *

from gi.repository import GObject
GObject.threads_init()
from gi.repository import TelepathyGLib as Tp

class NewContact(InnerWindow):
    def __init__(self, parent, am):
        self.parentwin = parent
        InnerWindow.__init__(self, parent)
        
        global log
        log = get_log()
        
        self.box = box = Box(self)

        acc_sel = Hoversel(self)
        acc_sel.text_set(_("Select account"))
        acc_sel.size_hint_weight_set(1.0, 0.0)
        acc_sel.size_hint_align_set(-1.0, 0.0)
        box.pack_end(acc_sel)
        acc_sel.show()
                        
        self.btn_box = btn_box = Box(self)
        btn_box.horizontal_set(True)
        btn_box.size_hint_weight_set(1.0, 0.0)
        btn_box.size_hint_align_set(-1.0, 0.0)
        box.pack_end(btn_box)
        btn_box.show()
        
        xbtn = Button(self)
        xbtn.text_set(_("Close"))
        #xbtn.callback_clicked_add(lambda x: parent.pager.content_pop())
        xbtn.callback_clicked_add(lambda x: self.delete())
        btn_box.pack_end(xbtn)
        xbtn.show()
        
        self.content_set(box)
        self.show()

        for account in am.get_valid_accounts():
            acc_sel.item_add(account.get_display_name(), account.get_icon_name(), ELM_ICON_STANDARD, self.selected, account)
            
    def selected(self, acc_sel, item, account):
        frame = Frame(self)
        frame.text_set("Identifier of contact")
        frame.size_hint_weight_set(1.0, 0.0)
        frame.size_hint_align_set(-1.0, 0.0)
        entry = Entry(self)
        entry.single_line_set(True)
        frame.content_set(entry)
        self.box.pack_before(frame, self.btn_box)
        entry.show()
        frame.show()
        
        go = Button(self)
        go.callback_clicked_add(self.add_go, entry, account)
        go.text_set(_("Add"))
        self.btn_box.pack_end(go)
        go.show()

    def add_go(self, obj, entry, account):
        identifier = entry.entry_get()
        account.request_presence_async(Tp.ConnectionPresenceType.AVAILABLE, "available", "", self.online, identifier)
        
    def online(self, account, result, identifier, *args):
        account.get_connection().get_contacts_by_id(
            (
                identifier,
            ),
            (
                Tp.ContactFeature.ALIAS,
                Tp.ContactFeature.CONTACT_GROUPS,
                Tp.ContactFeature.PRESENCE,
                Tp.ContactFeature.AVATAR_DATA,
                Tp.ContactFeature.SUBSCRIPTION_STATES,
                Tp.ContactFeature.CAPABILITIES,
                Tp.ContactFeature.CONTACT_INFO,
                Tp.ContactFeature.LOCATION,
            ),
            self.got_contact, None, None)

    def got_contact(self, connection, n_ids, contacts, requested_ids, failed_id_errors, error, user_data, *args):
        #import os, signal; os.kill(os.getpid(), signal.SIGINT)
        print("%s %s %s %s %s %s %s %s" % (connection, n_ids, contacts, requested_ids, failed_id_errors, error, user_data, args))
        for contact in contacts:
            print("Contact is %s" % contact)
