#!/usr/bin/env python

#########################################################################
#                                                                       #
# Apathy - A telepathy based communications client for mobile platforms #
# Copyright (C) 2009  Michael Sheldon <mike@mikeasoft.com>              #
# Copyright 2012 Kai Huuhko <kai@kaihuuhko.com>                         #
#                                                                       #
# This program is free software: you can redistribute it and/or modify  #
# it under the terms of the GNU General Public License as published by  #
# the Free Software Foundation, either version 3 of the License, or     #
# (at your option) any later version.                                   #
#                                                                       #
# This program is distributed in the hope that it will be useful,       #
# but WITHOUT ANY WARRANTY; without even the implied warranty of        #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
# GNU General Public License for more details.                          #
#                                                                       #
# You should have received a copy of the GNU General Public License     #
# along with this program.  If not, see <http://www.gnu.org/licenses/>. #
#                                                                       #
#########################################################################

import gettext
import logging

import ecore
from elementary import *
from evas import EVAS_ASPECT_CONTROL_VERTICAL

from Globals import *
from PresenceEditor import PresenceEditor

from gi.repository import GObject
GObject.threads_init()
from gi.repository import TelepathyGLib as Tp

class PresenceWidget(Window):
    def __init__(self):
        Window.__init__(self, "presence", ELM_WIN_BASIC)
        
        global log
        log = logging.getLogger("apathy.presence")

        self.title_set(_("Apathy - Presence"))
        self.callback_delete_request_add(self.quit)
        #self.alpha_set(True)

        bg = Background(self)
        bg.size_hint_weight_set(1.0, 1.0)
        self.resize_object_add(bg)
        bg.show()
        
        self.presence_list = presence_list = Hoversel(self)
        presence_list.size_hint_weight_set(1.0, 1.0)
        presence_list.size_hint_align_set(-1.0, 0.0)
        self._populate_presence_list()
        
        box = Box(self)
        box.size_hint_weight_set(1.0, 1.0)
        self.resize_object_add(box)
        box.pack_end(presence_list)
        box.show()
        presence_list.show()
        
        self.resize(300, 400)
        self.show()

        self.am = am = Tp.AccountManager.dup()
        factory = am.get_factory()
        factory.add_account_features([Tp.Account.get_feature_quark_connection()])
        factory.add_connection_features([
            Tp.Connection.get_feature_quark_contact_list(),
            Tp.Connection.get_feature_quark_contact_groups(),
            Tp.Connection.get_feature_quark_avatar_requirements(),
        ])

        am.prepare_async(None, self._am_ready, None)
        
    def _am_ready(self, am, result, error):
        am.prepare_finish(result)
        am.connect("most-available-presence-changed", self.presence_changed)
        presence, status, message = am.get_most_available_presence()
        self.presence_list.text_set(status.capitalize())
        ic = Icon(self)
        ic.standard_set("user-"+status)
        self.presence_list.content_set(ic)

    def _populate_presence_list(self):
        presences = {
            "offline": (1, True, False),
            "available": (2, True, True),
            "away": (3, True, True),
            "xa": (4, True, True),
            "hidden": (5, True, False),
            "dnd": (6, True, True),
            "unknown": (7, False, False),
            "error": (8, False, False),
        }
            
        conf = get_conf()

        for status, spec in presences.iteritems():
            presence, settable_on_self, can_have_message = spec
            if settable_on_self:
                if can_have_message:
                    try:
                        message = conf.get("StatusMessages", status)
                    except:
                        log.exception("Status message for status %s not found in configuration file." % (status))
                        message = ""
                if status == "xa":
                    icon = "user-away-extended"
                elif status == "hidden":
                    icon = "user-invisible"
                elif status == "dnd":
                    icon = "user-busy"
                else:
                    icon = "user-"+status
                self.presence_list.item_add(status.capitalize(), icon, ELM_ICON_STANDARD, self._set_presence, presence, status, message)
                    
        #self.presence_list.item_add(_("Edit")+"...", None, 0, self._edit_presences)

    def _set_presence(self, obj, event, presence, status, message=""):
        self.am.set_all_requested_presences(presence, status, message)

    def _edit_presences(self, obj, event, *args):
        editor = PresenceEditor(self)
        self.pager.item_simple_push(editor)
        
    def presence_changed(self, am, presence, status, message):
        self.presence_list.text_set(status.capitalize())
        ic = Icon(self)
        ic.standard_set("user-"+status)
        self.presence_list.content_set(ic)
        
    def quit(self, *args):
        exit()

if __name__ == "__main__":
    gettext.install("apathy", "/usr/share/locale", unicode=1)

    # Log
    if not os.path.exists(USER_PATH):
        os.mkdir(USER_PATH, 0700)
    log = logging.getLogger("apathy.presence")
    log.setLevel(logging.DEBUG)
    streamformatter = logging.Formatter("[%(levelname)s] in %(module)s line %(lineno)s (%(funcName)s): %(message)s")
    handler_stream = logging.StreamHandler()
    handler_stream.setFormatter(streamformatter)
    log.addHandler(handler_stream)

    init()

    ecore.main_loop_glib_integrate()
    presence = PresenceWidget()
    
    run()
    shutdown()
